@@ -1,9 +1,11 @@
 package Sidef::Types::Block::Block {
 
+    use utf8;
     use 5.016;
     use parent qw(Sidef::Object::Object);
 
     use List::Util qw();
+    use Sidef::Types::Number::Number;
 
     use overload
       q{bool} => sub { 1 },
@@ -374,7 +376,15 @@ package Sidef::Types::Block::Block {
 
         if ($ref eq 'Sidef::Types::Number::Number') {
             my ($type, $str) = $obj->_dump();
-            return scalar {dump => ($ref . "->_set_str('$type', '$str')"),};
+
+            if ($type eq 'int' and $str >= 0 and $str < Sidef::Types::Number::Number::ULONG_MAX) {
+                return scalar {dump => ($ref . "->_set_uint('$str')")};
+            }
+            elsif ($type eq 'int' and $str < 0 and $str > Sidef::Types::Number::Number::LONG_MIN) {
+                return scalar {dump => ($ref . "->_set_int('$str')")};
+            }
+
+            return scalar {dump => ($ref . "->_set_str('$type', '$str')")};
         }
         elsif ($ref eq 'Sidef::Types::Block::Block') {
             die "[ERROR] Blocks cannot be serialized by Block.ffork()!";
