@@ -1,5 +1,6 @@
 package Sidef::Types::Range::RangeNumber {
 
+    use utf8;
     use 5.014;
 
     use parent qw(
@@ -233,6 +234,42 @@ package Sidef::Types::Range::RangeNumber {
         $prod;
     }
 
+    sub bsearch {
+        my ($self, $block) = @_;
+
+        if ($self->{step}->abs->is_one) {
+            my $left  = Sidef::Types::Number::Number->new($self->{from});
+            my $right = Sidef::Types::Number::Number->new($self->{to});
+            return Sidef::Types::Number::Number::bsearch($left, $right, $block);
+        }
+
+        $self->to_a->bsearch($block);
+    }
+
+    sub bsearch_le {
+        my ($self, $block) = @_;
+
+        if ($self->{step}->abs->is_one) {
+            my $left  = Sidef::Types::Number::Number->new($self->{from});
+            my $right = Sidef::Types::Number::Number->new($self->{to});
+            return Sidef::Types::Number::Number::bsearch_le($left, $right, $block);
+        }
+
+        $self->to_a->bsearch_le($block);
+    }
+
+    sub bsearch_ge {
+        my ($self, $block) = @_;
+
+        if ($self->{step}->abs->is_one) {
+            my $left  = Sidef::Types::Number::Number->new($self->{from});
+            my $right = Sidef::Types::Number::Number->new($self->{to});
+            return Sidef::Types::Number::Number::bsearch_ge($left, $right, $block);
+        }
+
+        $self->to_a->bsearch_ge($block);
+    }
+
     sub dump {
         my ($self) = @_;
         Sidef::Types::String::String->new("$self");
